/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#ifndef ARGPARSER_H_INCLUDED
#define ARGPARSER_H_INCLUDED

#include <string>
#include <istream>
#include <ext/hash_map>

struct Arg;

struct hashstr 
{
  unsigned operator() (const string& a) const {
    struct hash<char *> strhash;
    return strhash(a.c_str());
  }
};

class ArgParser
{
 public:
  ArgParser() { }
  ~ArgParser() { }

  typedef hash_map<string, struct Arg *, hashstr > ArgMap;

  void AddArg(struct Arg *a);
  void Parse(istream& line, unsigned lineNum);
  ArgMap& Results() { return Args; }

 protected:
  ArgMap Args;
};

struct Arg
{
  Arg(const char *argName, ArgParser& ap) : name(argName) { ap.AddArg(this); }
  virtual ~Arg() { }
  virtual void Parse(istream& is)=0;
  string name;
};

template<class T> struct TArg : public Arg
{
  // XXXacm - should be able to configurably check bounds
  TArg(const char *s, T defval, ArgParser& ap) : Arg(s, ap), val(defval) { } 
  void Parse(istream& is) { 
    is >> val; 
  }
  T val;
};

typedef TArg<double> DoubleArg;
typedef TArg<string> StringArg;
typedef TArg<unsigned> UnsignedArg;



#endif // ARGPARSER_H_INCLUDED
