/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#ifndef EVENT_H_INCLUDED
#define EVENT_H_INCLUDED

#include <assert.h>
#include "global.h"

class Event
{
 public:
  Event(const char *name, double when) : Name(name), When(when), Dummy(false) { 
    assert(when >= GetCurrentTime());
  }

  // only class Simulator should ever delete an even once it's been
  // enqueued!
  virtual ~Event() { }

  // system will call this at CurrentTime == when
  virtual void Callback() = 0;

  const char *GetName() const { return Name; }
  double GetWhen() const { return When; }

  // call this to cancel the event.  when an event is cancelled, its
  // Callback will never be called.  the event will be deleted, but
  // not right away.
  void CancelEvent() { Dummy = true; }

  bool IsCancelled() { return Dummy; }

 protected:
  const char *Name;
  const double When;
  bool Dummy;
};

struct gtevent {
  bool operator() (const Event *a, const Event *b) const
  {
    return a->GetWhen() > b->GetWhen();
  }
};


#endif // EVENT_H_INCLUDED
