/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#include "global.h"
#include "Timer.h"
#include "Simulator.h"

Timer::Timer(const char *name, double offset, double period, Simulator *sim) : 
  Name(name), Period(period), simulator(sim), CurrentEv(NULL)
{
  assert(offset < Period);
  assert(offset >= 0);
  NewEvent(Period+offset);
}

Timer::~Timer()
{
  if (CurrentEv) {
    CurrentEv->CancelEvent();
    // drop our pointer to currentEv.  class Simulator will take care of deleting it.
    CurrentEv = NULL;
  }
}

void Timer::NewEvent(double offset)
{
  assert(!CurrentEv);
  double when = GetCurrentTime() + offset;
  CurrentEv = new TimerEvent(this, when);
  simulator->AddEvent(CurrentEv);
}

void Timer::Action()
{
  CurrentEv = NULL;
  NewEvent(Period);

  Callback();
}

