/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#ifndef TIMER_H_INCLUDED
#define TIMER_H_INCLUDED

#include "Event.h"

class TimerEvent;
class Simulator;

// a timer that periodically calls its Callback function
class Timer
{
 public:
  Timer(const char *name, double offset, double period, Simulator *sim);
  virtual ~Timer();

  // users of class Timer should subclass it and overload the Callback
  // method
  virtual void Callback() = 0;

 protected:
  const char *Name;
  const double Period;
  Simulator *simulator;
  TimerEvent *CurrentEv;

  void NewEvent(double when);
  void Action(); 
  friend class TimerEvent;
};


class TimerEvent : public Event
{
 public:
  TimerEvent(Timer *tim, double when) : Event("TimerEvent", when), Owner(tim) { }
  ~TimerEvent() { }

  void Callback() { Owner->Action(); }

 protected:
  Timer *Owner;
};

#endif // TIMER_H_INCLUDED
